searchData={"items":[{"type":"module","title":"CliMate","doc":"","ref":"CliMate.html"},{"type":"module","title":"CliMate.CLI","doc":"This module provides the CLI tools defined by `CliMate` without needing to\n`use CliMate` in you own project.\n\nNote that this module would not be available for a mix task distributed with\nhex.","ref":"CliMate.CLI.html"},{"type":"function","title":"CliMate.CLI.color/2","doc":"Outputs `iodata` in the current shell, wrapped with formatting\ninformation, such as `[color, iodata, :default_color]`.\n\n`color` should be a `IO.ANSI.format/2` compatible atom.","ref":"CliMate.CLI.html#color/2"},{"type":"function","title":"CliMate.CLI.debug/1","doc":"Outputs `iodata` in the current shell, formatted with cyan color.","ref":"CliMate.CLI.html#debug/1"},{"type":"function","title":"CliMate.CLI.error/1","doc":"Outputs `iodata` to `stderr` in the current shell, formatted with bright\nred color.","ref":"CliMate.CLI.html#error/1"},{"type":"function","title":"CliMate.CLI.format_usage/2","doc":"Returns a standard \"usage\" documentation block describing the different\noptions of the given command.","ref":"CliMate.CLI.html#format_usage/2"},{"type":"function","title":"Options - CliMate.CLI.format_usage/2","doc":"* `:format` - If `:moduledoc`, the formatted usage will be compatible for\n  embedding in a `@moduledoc` attribute. Any other value will generate a\n  simple terminal styled text. Defaults to `:cli`.","ref":"CliMate.CLI.html#format_usage/2-options"},{"type":"function","title":"CliMate.CLI.halt/1","doc":"Stops the execution of the Erlang runtime, with a given return code.\n\nIf not provided, the return code will be `0`.","ref":"CliMate.CLI.html#halt/1"},{"type":"function","title":"CliMate.CLI.halt_error/2","doc":"Combines `error/1` then `halt/1`. Halts the Erlang runtime with a `1`\nreturn code by default.","ref":"CliMate.CLI.html#halt_error/2"},{"type":"function","title":"CliMate.CLI.halt_success/1","doc":"Combines `success/1` then `halt/1`. Halts the Erlang runtime with a `0`\nreturn code.","ref":"CliMate.CLI.html#halt_success/1"},{"type":"function","title":"CliMate.CLI.parse/2","doc":"Accepts the command line arguments and the definition of a command\n(options, arguments, and metadata) and returns a parse result with values\nextracted from the command line arguments.","ref":"CliMate.CLI.html#parse/2"},{"type":"function","title":"Defining options - CliMate.CLI.parse/2","doc":"Options definitions is a `Keyword` whose keys are the option name, and\nvalues are the options parameters. Note that keys with underscores like\n`some_thing` define options in kebab case like `--some-thing`.\n\nThe following parameters are available:\n\n- `:type` - Can be either `:boolean`, `:integer` or `:string`. The default\n  value is `:string`.\n- `:short` - Defines the shortcut for the option, for instance `-p`\n  instead of `--port`.\n- `:default` - Defines the default value if the corresponding option is\n  not defined in `argv`.See \"Default values\" below.\n- `:doc` - Accepts a string that will be used when formatting the usage\n  block for the command.\n\nNote that the `:help` option is always defined and cannot be overridden.","ref":"CliMate.CLI.html#parse/2-defining-options"},{"type":"function","title":"Default values - CliMate.CLI.parse/2","doc":"Default values can be omitted, in that case, the option will not be\npresent at all if not provided in the command line.\n\nWhen defined, a default value can be:\n\n* A raw value, that is anything that is not a function. This value will be\n  used as the default value.\n* A function of arity zero. This function will be called when the option\n  is not provided in the command line and the result value will be used as\n  the default value. For instance `fn -> 123 end` or `&default_age/0`.\n* A function of arity one. This function will be called with the option\n  key as its argument. For instance, passing `&default_opt/1` as the\n  `:default` for an option definition allow to define the following\n  function:\n\n      defp default_opt(:port), do: 4000\n      defp default_opt(:scheme), do: \"http\"","ref":"CliMate.CLI.html#parse/2-default-values"},{"type":"function","title":"Options examples - CliMate.CLI.parse/2","doc":"iex> {:ok, result} = parse(~w(--who joe), [options: [who: [type: :string]]])\n    iex> result.options.who\n    \"joe\"\n\n    iex> {:ok, result} = parse(~w(--who joe), [options: [who: []]])\n    iex> result.options.who\n    \"joe\"\n\n    iex> {:ok, result} = parse(~w(--port 4000), [options: [port: [type: :integer]]])\n    iex> result.options.port\n    4000\n\n    iex> {:ok, result} = parse(~w(-p 4000), [options: [port: [type: :integer, short: :p]]])\n    iex> result.options.port\n    4000\n\n    iex> parse(~w(--port nope), [options: [port: [type: :integer]]])\n    {:error, {:invalid, [{\"--port\", \"nope\"}]}}\n\n    iex> {:ok, result} = parse([], [options: [lang: [default: \"elixir\"]]])\n    iex> result.options.lang\n    \"elixir\"\n\n    iex> {:ok, result} = parse([], [options: [lang: []]])\n    iex> Map.has_key?(result.options, :lang)\n    false\n\n    iex> {:ok, result} = parse([], options: [])\n    iex> result.options.help\n    false\n\n    iex> {:ok, result} = parse(~w(--help), options: [])\n    iex> result.options.help\n    true","ref":"CliMate.CLI.html#parse/2-options-examples"},{"type":"function","title":"Defining arguments - CliMate.CLI.parse/2","doc":"Arguments can be defined in the same way, providing a `Keyword` where the\nkeys are the argument names and the values are the parameters.","ref":"CliMate.CLI.html#parse/2-defining-arguments"},{"type":"function","title":"Defining arguments - CliMate.CLI.parse/2","doc":"The following parameters are available:\n\n- `:required` - A boolean marking the argument as required. **Note that\n  arguments are required by default**. Keys for optional arguments that\n  are not provided by the command line will not be defined in the results.\n- `:cast` - Accepts a fun or a `{module, function, arguments}` tuple to\n  transform the argument value when parsing. The invoked function must\n  return a result tuple: `{:ok, _} | {:error, _}`.\n- `:doc` - Accepts a string that will be used when formatting the usage\n  block for the command. Note that this is not currently implemented for\n  arguments.","ref":"CliMate.CLI.html#parse/2-defining-arguments"},{"type":"function","title":"Arguments examples - CliMate.CLI.parse/2","doc":"iex> {:ok, result} = parse(~w(joe), arguments: [who: []])\n    iex> result.arguments.who\n    \"joe\"\n\n    iex> parse([], arguments: [who: []])\n    {:error, {:missing_argument, :who}}\n\n    iex> {:ok, result} = parse([], arguments: [who: [required: false]])\n    iex> result.arguments\n    %{}\n\n    iex> cast = fn string -> Date.from_iso8601(string) end\n    iex> {:ok, result} = parse([\"2022-12-22\"], arguments: [date: [cast: cast]])\n    iex> result.arguments.date\n    ~D[2022-12-22]\n\n    iex> cast = {Date, :from_iso8601, []}\n    iex> {:ok, result} = parse([\"2022-12-22\"], arguments: [date: [cast: cast]])\n    iex> result.arguments.date\n    ~D[2022-12-22]\n\n    iex> cast = {Date, :from_iso8601, []}\n    iex> parse([\"not-a-date\"], arguments: [date: [cast: cast]])\n    {:error, {:argument_cast, :date, :invalid_format}}","ref":"CliMate.CLI.html#parse/2-arguments-examples"},{"type":"function","title":"CliMate.CLI.parse_or_halt!/2","doc":"Attempts to parse the command line arguments `argv` with the defined\ncommand.\n\nCommand options and arguments are documented in the `parse/2` function of\nthis module.\n\nIn `parse_or_halt!/2`, the successful return value will not be wrapped in\nan `:ok` tuple, but directly a map with the `:options` and `:arguments`\nkeys.\n\nIn case of a parse error, this function will output the usage block\nfollowed by a formatted error message, and halt the Erlang runtime.","ref":"CliMate.CLI.html#parse_or_halt!/2"},{"type":"function","title":"CliMate.CLI.put_shell/1","doc":"Defines the current shell to send console messages to. Accepts either\n`CliMate.CLI` or `CliMate.CLI.ProcessShell`.\n\nThe shell is saved to `:persistent_term`, so shell should not be changed\nrepeatedly during runtime. This method of persistence is subject to change\nand should not be relied on.","ref":"CliMate.CLI.html#put_shell/1"},{"type":"function","title":"CliMate.CLI.shell/0","doc":"Returns the current shell used by CliMate.CLI to send output.","ref":"CliMate.CLI.html#shell/0"},{"type":"function","title":"CliMate.CLI.success/1","doc":"Outputs `iodata` in the current shell, formatted with green color.","ref":"CliMate.CLI.html#success/1"},{"type":"function","title":"CliMate.CLI.warn/1","doc":"Outputs `iodata` to `stderr` in the current shell, formatted with yellow\ncolor.","ref":"CliMate.CLI.html#warn/1"},{"type":"function","title":"CliMate.CLI.writeln/1","doc":"Outputs `iodata` in the current shell.","ref":"CliMate.CLI.html#writeln/1"},{"type":"module","title":"CliMate.CLI.ProcessShell","doc":"An output shell implementation allowing your CLI commands to send their\noutput as messages to themselves. This is most useful for tests.\n\nThe process that will receive the messages is found by looking up the\nfirst pid in the `:\"$callers\"` key of the process dictionary, or\n`self()` if there is no caller.\n\nUse `CliMate.CLI.put_shell(CliMate.CLI.ProcessShell)` to enable this\nshell.","ref":"CliMate.CLI.ProcessShell.html"},{"type":"function","title":"CliMate.CLI.ProcessShell.message_target/0","doc":"Returns the pid of the process that will receive output messages.","ref":"CliMate.CLI.ProcessShell.html#message_target/0"},{"type":"extras","title":"License","doc":"# The MIT License\n\nCopyright (c) 2022, Ludovic Demblans\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of\nthis software and associated documentation files (the \"Software\"), to deal in\nthe Software without restriction, including without limitation the rights to\nuse, copy, modify, merge, publish, distribute, sublicense, and/or sell copies\nof the Software, and to permit persons to whom the Software is furnished to do\nso, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"Overview","doc":"# CliMate\n\nThis package implements a simple wrapper around\n[OptionParser](https://hexdocs.pm/elixir/OptionParser.html) that allows to\ndefine options, documentation and arguments in one data structure.\n\nIt also provides classic printing of \"usage\" for a command.\n\nThis package is targeted at standalone mix commands, _i.e._ commands defined in\npackages installed globally (for instance `mix archive.install hex\nsome_package`). Those packages do not contain their dependencies, and for that\nreason, all code defined in CliMate is injected in the consumer code.\n\nFor this reason, CliMate only defines a small set of  features.","ref":"readme.html"},{"type":"extras","title":"Installation & Use - Overview","doc":"```elixir\ndef deps do\n  [\n    {:cli_mate, \"~> 0.4\", runtime: false},\n  ]\nend\n```\n\nAll available functions of this library are defined in `CliMate.CLI`.\n\nIf you want to use CLI Mate in a mix task that you will distribute as a\nstandalone command, you must define a wrapper module:\n\n```elixir\ndefmodule App.CLI do\n  use CliMate\nend\n```\n\nThis module will export the exact same functionality as `CliMate.CLI`.\n\nMake sure to provide `runtime: false` in your dependency declaration for\n`:cli_mate`.","ref":"readme.html#installation-use"},{"type":"extras","title":"Providing usage block in `mix help my.command` - Overview","doc":"The default usage block rendered when calling `mix my.command --help` is not\nincluded by default when calling `mix help my.command`.\n\nIf you want, you can include it manually by adding it to the `@moduledoc`\nattribute:\n\n```elixir\n# Use the CLI module that you defined if you need to.\n\nalias App.CLI\n\n# Define the command once, so you can use it in module docs and in the `run/1`\n# callback.\n\n@command [\n  module: __MODULE__,\n  options: [\n    some_option: [\n      type: :boolean,\n      short: :s,\n      doc: \"Does something, for sure.\"\n    ]\n  ]\n]\n\n# Generate the usage block and include it to @moduledoc, with the appropriate\n# format.\n\n@moduledoc \"\"\"\nThis command does things!\n\n#{CLI.format_usage(@command, format: :moduledoc)}\n\"\"\"\n\n# Since we defined the options in a @command attribute, we can now reuse it in\n# the code.\n\n@impl Mix.Task\ndef run(argv) do\n  %{options: options,arguments: arguments} = CLI.parse_or_halt!(argv, @command)\n  # ...\nend\n```","ref":"readme.html#providing-usage-block-in-mix-help-my-command"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,48]}}